@@ -1,13 +1,118 @@
-"""Error classes for CherryPy."""
+"""Exception classes for CherryPy.
+
+CherryPy provides (and uses) exceptions for declaring that the HTTP response
+should be a status other than the default "200 OK". You can ``raise`` them like
+normal Python exceptions. You can also call them and they will raise themselves;
+this means you can set an :class:`HTTPError<cherrypy._cperror.HTTPError>`
+or :class:`HTTPRedirect<cherrypy._cperror.HTTPRedirect>` as the
+:attr:`request.handler<cherrypy._cprequest.Request.handler>`.
+
+.. _redirectingpost:
+
+Redirecting POST
+================
+
+When you GET a resource and are redirected by the server to another Location,
+there's generally no problem since GET is both a "safe method" (there should
+be no side-effects) and an "idempotent method" (multiple calls are no different
+than a single call).
+
+POST, however, is neither safe nor idempotent--if you
+charge a credit card, you don't want to be charged twice by a redirect!
+
+For this reason, *none* of the 3xx responses permit a user-agent (browser) to
+resubmit a POST on redirection without first confirming the action with the user:
+
+=====    =================================    ===========
+300      Multiple Choices                     Confirm with the user
+301      Moved Permanently                    Confirm with the user
+302      Found (Object moved temporarily)     Confirm with the user
+303      See Other                            GET the new URI--no confirmation
+304      Not modified                         (for conditional GET only--POST should not raise this error)
+305      Use Proxy                            Confirm with the user
+307      Temporary Redirect                   Confirm with the user
+=====    =================================    ===========
+
+However, browsers have historically implemented these restrictions poorly;
+in particular, many browsers do not force the user to confirm 301, 302
+or 307 when redirecting POST. For this reason, CherryPy defaults to 303,
+which most user-agents appear to have implemented correctly. Therefore, if
+you raise HTTPRedirect for a POST request, the user-agent will most likely
+attempt to GET the new URI (without asking for confirmation from the user).
+We realize this is confusing for developers, but it's the safest thing we
+could do. You are of course free to raise ``HTTPRedirect(uri, status=302)``
+or any other 3xx status if you know what you're doing, but given the
+environment, we couldn't let any of those be the default.
+
+Custom Error Handling
+=====================
+
+.. image:: /refman/cperrors.gif
+
+Anticipated HTTP responses
+--------------------------
+
+The 'error_page' config namespace can be used to provide custom HTML output for
+expected responses (like 404 Not Found). Supply a filename from which the output
+will be read. The contents will be interpolated with the values %(status)s,
+%(message)s, %(traceback)s, and %(version)s using plain old Python
+`string formatting <http://www.python.org/doc/2.6.4/library/stdtypes.html#string-formatting-operations>`_.
+
+::
+
+    _cp_config = {'error_page.404': os.path.join(localDir, "static/index.html")}
+
+
+Beginning in version 3.1, you may also provide a function or other callable as
+an error_page entry. It will be passed the same status, message, traceback and
+version arguments that are interpolated into templates::
+
+    def error_page_402(status, message, traceback, version):
+        return "Error %s - Well, I'm very sorry but you haven't paid!" % status
+    cherrypy.config.update({'error_page.402': error_page_402})
+
+Also in 3.1, in addition to the numbered error codes, you may also supply
+"error_page.default" to handle all codes which do not have their own error_page entry.
+
+
+
+Unanticipated errors
+--------------------
+
+CherryPy also has a generic error handling mechanism: whenever an unanticipated
+error occurs in your code, it will call
+:func:`Request.error_response<cherrypy._cprequest.Request.error_response>` to set
+the response status, headers, and body. By default, this is the same output as
+:class:`HTTPError(500) <cherrypy._cperror.HTTPError>`. If you want to provide
+some other behavior, you generally replace "request.error_response".
+
+Here is some sample code that shows how to display a custom error message and
+send an e-mail containing the error::
+
+    from cherrypy import _cperror
+
+    def handle_error():
+        cherrypy.response.status = 500
+        cherrypy.response.body = ["<html><body>Sorry, an error occured</body></html>"]
+        sendMail('error@domain.com', 'Error in your web app', _cperror.format_exc())
+
+    class Root:
+        _cp_config = {'request.error_response': handle_error}
+
+
+Note that you have to explicitly set :attr:`response.body <cherrypy._cprequest.Response.body>`
+and not simply return an error message as a result.
+"""
 
 from cgi import escape as _escape
 from sys import exc_info as _exc_info
 from traceback import format_exception as _format_exception
-from urlparse import urljoin as _urljoin
-from cherrypy.lib import http as _http
+from cherrypy._cpcompat import basestring, iteritems, urljoin as _urljoin
+from cherrypy.lib import httputil as _httputil
 
 
 class CherryPyException(Exception):
+    """A base class for CherryPy exceptions."""
     pass
 
 
@@ -19,14 +124,16 @@ class TimeoutError(CherryPyException):
 class InternalRedirect(CherryPyException):
     """Exception raised to switch to the handler for a different URL.
     
-    Any request.params must be supplied in a query string.
+    This exception will redirect processing to another path within the site
+    (without informing the client). Provide the new path as an argument when
+    raising the exception. Provide any params in the querystring for the new URL.
     """
     
-    def __init__(self, path):
+    def __init__(self, path, query_string=""):
         import cherrypy
-        request = cherrypy.request
+        self.request = cherrypy.serving.request
         
-        self.query_string = ""
+        self.query_string = query_string
         if "?" in path:
             # Separate any params included in the path
             path, self.query_string = path.split("?", 1)
@@ -35,7 +142,7 @@ class InternalRedirect(CherryPyException):
         #  1. a URL relative to root (e.g. "/dummy")
         #  2. a URL relative to the current path
         # Note that any query string will be discarded.
-        path = _urljoin(request.path_info, path)
+        path = _urljoin(self.request.path_info, path)
         
         # Set a 'path' member attribute so that code which traps this
         # error can have access to it.
@@ -47,21 +154,49 @@ class InternalRedirect(CherryPyException):
 class HTTPRedirect(CherryPyException):
     """Exception raised when the request should be redirected.
     
+    This exception will force a HTTP redirect to the URL or URL's you give it.
     The new URL must be passed as the first argument to the Exception,
-    e.g., HTTPRedirect(newUrl). Multiple URLs are allowed. If a URL is
-    absolute, it will be used as-is. If it is relative, it is assumed
-    to be relative to the current cherrypy.request.path_info.
+    e.g., HTTPRedirect(newUrl). Multiple URLs are allowed in a list.
+    If a URL is absolute, it will be used as-is. If it is relative, it is
+    assumed to be relative to the current cherrypy.request.path_info.
+
+    If one of the provided URL is a unicode object, it will be encoded
+    using the default encoding or the one passed in parameter.
+    
+    There are multiple types of redirect, from which you can select via the
+    ``status`` argument. If you do not provide a ``status`` arg, it defaults to
+    303 (or 302 if responding with HTTP/1.0).
+    
+    Examples::
+    
+        raise cherrypy.HTTPRedirect("")
+        raise cherrypy.HTTPRedirect("/abs/path", 307)
+        raise cherrypy.HTTPRedirect(["path1", "path2?a=1&b=2"], 301)
+    
+    See :ref:`redirectingpost` for additional caveats.
     """
     
-    def __init__(self, urls, status=None):
+    status = None
+    """The integer HTTP status code to emit."""
+    
+    urls = None
+    """The list of URL's to emit."""
+
+    encoding = 'utf-8'
+    """The encoding when passed urls are unicode objects"""
+    
+    def __init__(self, urls, status=None, encoding=None):
         import cherrypy
-        request = cherrypy.request
+        request = cherrypy.serving.request
         
         if isinstance(urls, basestring):
             urls = [urls]
         
         abs_urls = []
         for url in urls:
+            if isinstance(url, unicode):
+                url = url.encode(encoding or self.encoding)
+                
             # Note that urljoin will "do the right thing" whether url is:
             #  1. a complete URL with host (e.g. "http://www.example.com/test")
             #  2. a URL relative to root (e.g. "/dummy")
@@ -73,7 +208,7 @@ class HTTPRedirect(CherryPyException):
         
         # RFC 2616 indicates a 301 response code fits our goal; however,
         # browser support for 301 is quite messy. Do 302/303 instead. See
-        # http://ppewww.ph.gla.ac.uk/~flavell/www/post-redirect.html
+        # http://www.alanflavell.org.uk/www/post-redirect.html
         if status is None:
             if request.protocol >= (1, 1):
                 status = 303
@@ -94,11 +229,11 @@ class HTTPRedirect(CherryPyException):
         HTTPRedirect object and set its output without *raising* the exception.
         """
         import cherrypy
-        response = cherrypy.response
+        response = cherrypy.serving.response
         response.status = status = self.status
         
         if status in (300, 301, 302, 303, 307):
-            response.headers['Content-Type'] = "text/html"
+            response.headers['Content-Type'] = "text/html;charset=utf-8"
             # "The ... URI SHOULD be given by the Location field
             # in the response."
             response.headers['Location'] = self.urls[0]
@@ -112,7 +247,8 @@ class HTTPRedirect(CherryPyException):
                    303: "This resource can be found at <a href='%s'>%s</a>.",
                    307: "This resource has moved temporarily to <a href='%s'>%s</a>.",
                    }[status]
-            response.body = "<br />\n".join([msg % (u, u) for u in self.urls])
+            msgs = [msg % (u, u) for u in self.urls]
+            response.body = "<br />\n".join(msgs)
             # Previous code may have set C-L, so we have to reset it
             # (allow finalize to set it).
             response.headers.pop('Content-Length', None)
@@ -153,7 +289,7 @@ def clean_headers(status):
     """Remove any headers which should not apply to an error response."""
     import cherrypy
     
-    response = cherrypy.response
+    response = cherrypy.serving.response
     
     # Remove headers which applied to the original content,
     # but do not apply to the error page.
@@ -161,7 +297,7 @@ def clean_headers(status):
     for key in ["Accept-Ranges", "Age", "ETag", "Location", "Retry-After",
                 "Vary", "Content-Encoding", "Content-Length", "Expires",
                 "Content-Location", "Content-MD5", "Last-Modified"]:
-        if respheaders.has_key(key):
+        if key in respheaders:
             del respheaders[key]
     
     if status != 416:
@@ -171,23 +307,49 @@ def clean_headers(status):
         # specifies the current length of the selected resource.
         # A response with status code 206 (Partial Content) MUST NOT
         # include a Content-Range field with a byte-range- resp-spec of "*".
-        if respheaders.has_key("Content-Range"):
+        if "Content-Range" in respheaders:
             del respheaders["Content-Range"]
 
 
 class HTTPError(CherryPyException):
-    """ Exception used to return an HTTP error code (4xx-5xx) to the client.
-        This exception will automatically set the response status and body.
+    """Exception used to return an HTTP error code (4xx-5xx) to the client.
+    
+    This exception can be used to automatically send a response using a http status
+    code, with an appropriate error page. It takes an optional
+    ``status`` argument (which must be between 400 and 599); it defaults to 500
+    ("Internal Server Error"). It also takes an optional ``message`` argument,
+    which will be returned in the response body. See
+    `RFC 2616 <http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html#sec10.4>`_
+    for a complete list of available error codes and when to use them.
+    
+    Examples::
         
-        A custom message (a long description to display in the browser)
-        can be provided in place of the default.
+        raise cherrypy.HTTPError(403)
+        raise cherrypy.HTTPError("403 Forbidden", "You are not allowed to access this resource.")
     """
     
+    status = None
+    """The HTTP status code. May be of type int or str (with a Reason-Phrase)."""
+    
+    code = None
+    """The integer HTTP status code."""
+    
+    reason = None
+    """The HTTP Reason-Phrase string."""
+    
     def __init__(self, status=500, message=None):
-        self.status = status = int(status)
-        if status < 400 or status > 599:
+        self.status = status
+        try:
+            self.code, self.reason, defaultmsg = _httputil.valid_status(status)
+        except ValueError, x:
+            raise self.__class__(500, x.args[0])
+        
+        if self.code < 400 or self.code > 599:
             raise ValueError("status must be between 400 and 599.")
-        self.message = message
+        
+        # See http://www.python.org/dev/peps/pep-0352/
+        # self.message = message
+        self._message = message or defaultmsg
         CherryPyException.__init__(self, status, message)
     
     def set_response(self):
@@ -198,24 +360,24 @@ class HTTPError(CherryPyException):
         """
         import cherrypy
         
-        response = cherrypy.response
+        response = cherrypy.serving.response
         
-        clean_headers(self.status)
+        clean_headers(self.code)
         
         # In all cases, finalize will be called after this method,
         # so don't bother cleaning up response values here.
         response.status = self.status
         tb = None
-        if cherrypy.request.show_tracebacks:
+        if cherrypy.serving.request.show_tracebacks:
             tb = format_exc()
-        response.headers['Content-Type'] = "text/html"
+        response.headers['Content-Type'] = "text/html;charset=utf-8"
+        response.headers.pop('Content-Length', None)
         
         content = self.get_error_page(self.status, traceback=tb,
-                                      message=self.message)
+                                      message=self._message)
         response.body = content
-        response.headers['Content-Length'] = len(content)
         
-        _be_ie_unfriendly(self.status)
+        _be_ie_unfriendly(self.code)
     
     def get_error_page(self, *args, **kwargs):
         return get_error_page(*args, **kwargs)
@@ -226,14 +388,19 @@ class HTTPError(CherryPyException):
 
 
 class NotFound(HTTPError):
-    """Exception raised when a URL could not be mapped to any handler (404)."""
+    """Exception raised when a URL could not be mapped to any handler (404).
+    
+    This is equivalent to raising
+    :class:`HTTPError("404 Not Found") <cherrypy._cperror.HTTPError>`.
+    """
     
     def __init__(self, path=None):
         if path is None:
             import cherrypy
-            path = cherrypy.request.script_name + cherrypy.request.path_info
+            request = cherrypy.serving.request
+            path = request.script_name + request.path_info
         self.args = (path,)
-        HTTPError.__init__(self, 404, "The path %r was not found." % path)
+        HTTPError.__init__(self, 404, "The path '%s' was not found." % path)
 
 
 _HTTPErrorTemplate = '''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
@@ -274,7 +441,7 @@ def get_error_page(status, **kwargs):
     import cherrypy
     
     try:
-        code, reason, message = _http.valid_status(status)
+        code, reason, message = _httputil.valid_status(status)
     except ValueError, x:
         raise cherrypy.HTTPError(500, x.args[0])
     
@@ -289,21 +456,21 @@ def get_error_page(status, **kwargs):
     if kwargs.get('version') is None:
         kwargs['version'] = cherrypy.__version__
     
-    for k, v in kwargs.iteritems():
+    for k, v in iteritems(kwargs):
         if v is None:
             kwargs[k] = ""
         else:
             kwargs[k] = _escape(kwargs[k])
     
     # Use a custom template or callable for the error page?
-    pages = cherrypy.request.error_page
+    pages = cherrypy.serving.request.error_page
     error_page = pages.get(code) or pages.get('default')
     if error_page:
         try:
-            if callable(error_page):
+            if hasattr(error_page, '__call__'):
                 return error_page(**kwargs)
             else:
-                return file(error_page, 'rb').read() % kwargs
+                return open(error_page, 'rb').read() % kwargs
         except:
             e = _format_exception(*_exc_info())[-1]
             m = kwargs['message']
@@ -324,7 +491,7 @@ _ie_friendly_error_sizes = {
 
 def _be_ie_unfriendly(status):
     import cherrypy
-    response = cherrypy.response
+    response = cherrypy.serving.response
     
     # For some statuses, Internet Explorer 5+ shows "friendly error
     # messages" instead of our response.body if the body is smaller
@@ -343,7 +510,7 @@ def _be_ie_unfriendly(status):
             # in one chunk or it will still get replaced! Bah.
             content = content + (" " * (s - l))
         response.body = content
-        response.headers['Content-Length'] = len(content)
+        response.headers['Content-Length'] = str(len(content))
 
 
 def format_exc(exc=None):
@@ -374,6 +541,8 @@ def bare_error(extrabody=None):
     
     body = "Unrecoverable error in the server."
     if extrabody is not None:
+        if not isinstance(extrabody, str): 
+            extrabody = extrabody.encode('utf-8')
         body += "\n" + extrabody
     
     return ("500 Internal Server Error",
