@@ -1,16 +1,16 @@
 """Functions for builtin CherryPy tools."""
 
 import logging
-import md5
 import re
 
 import cherrypy
-from cherrypy.lib import http as _http
+from cherrypy._cpcompat import basestring, ntob, md5, set
+from cherrypy.lib import httputil as _httputil
 
 
 #                     Conditional HTTP request support                     #
 
-def validate_etags(autotags=False):
+def validate_etags(autotags=False, debug=False):
     """Validate the current ETag against If-Match, If-None-Match headers.
     
     If autotags is True, an ETag response-header value will be provided
@@ -24,42 +24,62 @@ def validate_etags(autotags=False):
     use for entity tags in a possibly destructive fashion. Likewise, if you
     raise 304 Not Modified, the response body will be empty, the ETag hash
     will be incorrect, and your application will break.
-    See http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.24
+    See :rfc:`2616` Section 14.24.
     """
-    response = cherrypy.response
+    response = cherrypy.serving.response
     
     # Guard against being run twice.
     if hasattr(response, "ETag"):
         return
     
-    status, reason, msg = _http.valid_status(response.status)
+    status, reason, msg = _httputil.valid_status(response.status)
     
     etag = response.headers.get('ETag')
     
     # Automatic ETag generation. See warning in docstring.
-    if (not etag) and autotags:
-        if status == 200:
-            etag = response.collapse_body()
-            etag = '"%s"' % md5.new(etag).hexdigest()
-            response.headers['ETag'] = etag
+    if etag:
+        if debug:
+            cherrypy.log('ETag already set: %s' % etag, 'TOOLS.ETAGS')
+    elif not autotags:
+        if debug:
+            cherrypy.log('Autotags off', 'TOOLS.ETAGS')
+    elif status != 200:
+        if debug:
+            cherrypy.log('Status not 200', 'TOOLS.ETAGS')
+    else:
+        etag = response.collapse_body()
+        etag = '"%s"' % md5(etag).hexdigest()
+        if debug:
+            cherrypy.log('Setting ETag: %s' % etag, 'TOOLS.ETAGS')
+        response.headers['ETag'] = etag
     
     response.ETag = etag
     
     # "If the request would, without the If-Match header field, result in
     # anything other than a 2xx or 412 status, then the If-Match header
     # MUST be ignored."
+    if debug:
+        cherrypy.log('Status: %s' % status, 'TOOLS.ETAGS')
     if status >= 200 and status <= 299:
-        request = cherrypy.request
+        request = cherrypy.serving.request
         
         conditions = request.headers.elements('If-Match') or []
         conditions = [str(x) for x in conditions]
+        if debug:
+            cherrypy.log('If-Match conditions: %s' % repr(conditions),
+                         'TOOLS.ETAGS')
         if conditions and not (conditions == ["*"] or etag in conditions):
             raise cherrypy.HTTPError(412, "If-Match failed: ETag %r did "
                                      "not match %r" % (etag, conditions))
         
         conditions = request.headers.elements('If-None-Match') or []
         conditions = [str(x) for x in conditions]
+        if debug:
+            cherrypy.log('If-None-Match conditions: %s' % repr(conditions),
+                         'TOOLS.ETAGS')
         if conditions == ["*"] or etag in conditions:
+            if debug:
+                cherrypy.log('request.method: %s' % request.method, 'TOOLS.ETAGS')
             if request.method in ("GET", "HEAD"):
                 raise cherrypy.HTTPRedirect([], 304)
             else:
@@ -72,12 +92,12 @@ def validate_since():
     If no code has set the Last-Modified response header, then no validation
     will be performed.
     """
-    response = cherrypy.response
+    response = cherrypy.serving.response
     lastmod = response.headers.get('Last-Modified')
     if lastmod:
-        status, reason, msg = _http.valid_status(response.status)
+        status, reason, msg = _httputil.valid_status(response.status)
         
-        request = cherrypy.request
+        request = cherrypy.serving.request
         
         since = request.headers.get('If-Unmodified-Since')
         if since and since != lastmod:
@@ -95,8 +115,38 @@ def validate_since():
 
 #                                Tool code                                #
 
+def allow(methods=None, debug=False):
+    """Raise 405 if request.method not in methods (default GET/HEAD).
+    
+    The given methods are case-insensitive, and may be in any order.
+    If only one method is allowed, you may supply a single string;
+    if more than one, supply a list of strings.
+    
+    Regardless of whether the current method is allowed or not, this
+    also emits an 'Allow' response header, containing the given methods.
+    """
+    if not isinstance(methods, (tuple, list)):
+        methods = [methods]
+    methods = [m.upper() for m in methods if m]
+    if not methods:
+        methods = ['GET', 'HEAD']
+    elif 'GET' in methods and 'HEAD' not in methods:
+        methods.append('HEAD')
+    
+    cherrypy.response.headers['Allow'] = ', '.join(methods)
+    if cherrypy.request.method not in methods:
+        if debug:
+            cherrypy.log('request.method %r not in methods %r' %
+                         (cherrypy.request.method, methods), 'TOOLS.ALLOW')
+        raise cherrypy.HTTPError(405)
+    else:
+        if debug:
+            cherrypy.log('request.method %r in methods %r' %
+                         (cherrypy.request.method, methods), 'TOOLS.ALLOW')
+
+
 def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
-          scheme='X-Forwarded-Proto'):
+          scheme='X-Forwarded-Proto', debug=False):
     """Change the base URL (scheme://host[:port][/path]).
     
     For running a CP server behind Apache, lighttpd, or other HTTP server.
@@ -104,7 +154,8 @@ def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
     If you want the new request.base to include path info (not just the host),
     you must explicitly set base to the full base path, and ALSO set 'local'
     to '', so that the X-Forwarded-Host request header (which never includes
-    path info) does not override it.
+    path info) does not override it. Regardless, the value for 'base' MUST
+    NOT end in a slash.
     
     cherrypy.request.remote.ip (the IP address of the client) will be
     rewritten if the header specified by the 'remote' arg is valid.
@@ -112,10 +163,12 @@ def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
     want to rewrite remote.ip, set the 'remote' arg to an empty string.
     """
     
-    request = cherrypy.request
+    request = cherrypy.serving.request
     
     if scheme:
         s = request.headers.get(scheme, None)
+        if debug:
+            cherrypy.log('Testing scheme %r:%r' % (scheme, s), 'TOOLS.PROXY')
         if s == 'on' and 'ssl' in scheme.lower():
             # This handles e.g. webfaction's 'X-Forwarded-Ssl: on' header
             scheme = 'https'
@@ -126,9 +179,13 @@ def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
         scheme = request.base[:request.base.find("://")]
     
     if local:
-        base = request.headers.get(local, base)
+        lbase = request.headers.get(local, None)
+        if debug:
+            cherrypy.log('Testing local %r:%r' % (local, lbase), 'TOOLS.PROXY')
+        if lbase is not None:
+            base = lbase.split(',')[0]
     if not base:
-        port = cherrypy.request.local.port
+        port = request.local.port
         if port == 80:
             base = '127.0.0.1'
         else:
@@ -142,6 +199,8 @@ def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
     
     if remote:
         xff = request.headers.get(remote)
+        if debug:
+            cherrypy.log('Testing remote %r:%r' % (remote, xff), 'TOOLS.PROXY')
         if xff:
             if remote == 'X-Forwarded-For':
                 # See http://bob.pythonmac.org/archives/2005/09/23/apache-x-forwarded-for-caveat/
@@ -149,42 +208,64 @@ def proxy(base=None, local='X-Forwarded-Host', remote='X-Forwarded-For',
             request.remote.ip = xff
 
 
-def ignore_headers(headers=('Range',)):
+def ignore_headers(headers=('Range',), debug=False):
     """Delete request headers whose field names are included in 'headers'.
     
     This is a useful tool for working behind certain HTTP servers;
     for example, Apache duplicates the work that CP does for 'Range'
     headers, and will doubly-truncate the response.
     """
-    request = cherrypy.request
+    request = cherrypy.serving.request
     for name in headers:
         if name in request.headers:
+            if debug:
+                cherrypy.log('Ignoring request header %r' % name,
+                             'TOOLS.IGNORE_HEADERS')
             del request.headers[name]
 
 
-def response_headers(headers=None):
+def response_headers(headers=None, debug=False):
     """Set headers on the response."""
+    if debug:
+        cherrypy.log('Setting response headers: %s' % repr(headers),
+                     'TOOLS.RESPONSE_HEADERS')
     for name, value in (headers or []):
-        cherrypy.response.headers[name] = value
+        cherrypy.serving.response.headers[name] = value
 response_headers.failsafe = True
 
 
 def referer(pattern, accept=True, accept_missing=False, error=403,
-            message='Forbidden Referer header.'):
+            message='Forbidden Referer header.', debug=False):
     """Raise HTTPError if Referer header does/does not match the given pattern.
     
-    pattern: a regular expression pattern to test against the Referer.
-    accept: if True, the Referer must match the pattern; if False,
+    pattern
+        A regular expression pattern to test against the Referer.
+        
+    accept
+        If True, the Referer must match the pattern; if False,
         the Referer must NOT match the pattern.
-    accept_missing: if True, permit requests with no Referer header.
-    error: the HTTP error code to return to the client on failure.
-    message: a string to include in the response body on failure.
+
+    accept_missing
+        If True, permit requests with no Referer header.
+
+    error
+        The HTTP error code to return to the client on failure.
+        
+    message
+        A string to include in the response body on failure.
+    
     """
     try:
-        match = bool(re.match(pattern, cherrypy.request.headers['Referer']))
+        ref = cherrypy.serving.request.headers['Referer']
+        match = bool(re.match(pattern, ref))
+        if debug:
+            cherrypy.log('Referer %r matches %r' % (ref, pattern),
+                         'TOOLS.REFERER')
         if accept == match:
             return
     except KeyError:
+        if debug:
+            cherrypy.log('No Referer header', 'TOOLS.REFERER')
         if accept_missing:
             return
     
@@ -195,6 +276,7 @@ class SessionAuth(object):
     """Assert that the user is logged in."""
     
     session_key = "username"
+    debug = False
     
     def check_username_and_password(self, username, password):
         pass
@@ -213,7 +295,7 @@ class SessionAuth(object):
         pass
     
     def login_screen(self, from_page='..', username='', error_msg='', **kwargs):
-        return """<html><body>
+        return ntob("""<html><body>
 Message: %(error_msg)s
 <form method="post" action="do_login">
     Login: <input type="text" name="username" value="%(username)s" size="10" /><br />
@@ -222,20 +304,22 @@ Message: %(error_msg)s
     <input type="submit" />
 </form>
 </body></html>""" % {'from_page': from_page, 'username': username,
-                     'error_msg': error_msg}
+                     'error_msg': error_msg}, "utf-8")
     
     def do_login(self, username, password, from_page='..', **kwargs):
         """Login. May raise redirect, or return True if request handled."""
+        response = cherrypy.serving.response
         error_msg = self.check_username_and_password(username, password)
         if error_msg:
             body = self.login_screen(from_page, username, error_msg)
-            cherrypy.response.body = body
-            if cherrypy.response.headers.has_key("Content-Length"):
+            response.body = body
+            if "Content-Length" in response.headers:
                 # Delete Content-Length header so finalize() recalcs it.
-                del cherrypy.response.headers["Content-Length"]
+                del response.headers["Content-Length"]
             return True
         else:
-            cherrypy.session[self.session_key] = cherrypy.request.login = username
+            cherrypy.serving.request.login = username
+            cherrypy.session[self.session_key] = username
             self.on_login(username)
             raise cherrypy.HTTPRedirect(from_page or "/")
     
@@ -245,43 +329,70 @@ Message: %(error_msg)s
         username = sess.get(self.session_key)
         sess[self.session_key] = None
         if username:
-            cherrypy.request.login = None
+            cherrypy.serving.request.login = None
             self.on_logout(username)
         raise cherrypy.HTTPRedirect(from_page)
     
     def do_check(self):
         """Assert username. May raise redirect, or return True if request handled."""
         sess = cherrypy.session
-        request = cherrypy.request
+        request = cherrypy.serving.request
+        response = cherrypy.serving.response
         
         username = sess.get(self.session_key)
         if not username:
             sess[self.session_key] = username = self.anonymous()
+            if self.debug:
+                cherrypy.log('No session[username], trying anonymous', 'TOOLS.SESSAUTH')
         if not username:
-            cherrypy.response.body = self.login_screen(cherrypy.url(qs=request.query_string))
-            if cherrypy.response.headers.has_key("Content-Length"):
+            url = cherrypy.url(qs=request.query_string)
+            if self.debug:
+                cherrypy.log('No username, routing to login_screen with '
+                             'from_page %r' % url, 'TOOLS.SESSAUTH')
+            response.body = self.login_screen(url)
+            if "Content-Length" in response.headers:
                 # Delete Content-Length header so finalize() recalcs it.
-                del cherrypy.response.headers["Content-Length"]
+                del response.headers["Content-Length"]
             return True
-        cherrypy.request.login = username
+        if self.debug:
+            cherrypy.log('Setting request.login to %r' % username, 'TOOLS.SESSAUTH')
+        request.login = username
         self.on_check(username)
     
     def run(self):
-        request = cherrypy.request
+        request = cherrypy.serving.request
+        response = cherrypy.serving.response
+        
         path = request.path_info
         if path.endswith('login_screen'):
+            if self.debug:
+                cherrypy.log('routing %r to login_screen' % path, 'TOOLS.SESSAUTH')
             return self.login_screen(**request.params)
         elif path.endswith('do_login'):
+            if request.method != 'POST':
+                response.headers['Allow'] = "POST"
+                if self.debug:
+                    cherrypy.log('do_login requires POST', 'TOOLS.SESSAUTH')
+                raise cherrypy.HTTPError(405)
+            if self.debug:
+                cherrypy.log('routing %r to do_login' % path, 'TOOLS.SESSAUTH')
             return self.do_login(**request.params)
         elif path.endswith('do_logout'):
+            if request.method != 'POST':
+                response.headers['Allow'] = "POST"
+                raise cherrypy.HTTPError(405)
+            if self.debug:
+                cherrypy.log('routing %r to do_logout' % path, 'TOOLS.SESSAUTH')
             return self.do_logout(**request.params)
         else:
+            if self.debug:
+                cherrypy.log('No special path, running do_check', 'TOOLS.SESSAUTH')
             return self.do_check()
 
 
 def session_auth(**kwargs):
     sa = SessionAuth()
-    for k, v in kwargs.iteritems():
+    for k, v in kwargs.items():
         setattr(sa, k, v)
     return sa.run()
 session_auth.__doc__ = """Session authentication hook.
@@ -293,59 +404,69 @@ to this function:
                  for k in dir(SessionAuth) if not k.startswith("__")])
 
 
-def log_traceback(severity=logging.DEBUG):
+def log_traceback(severity=logging.ERROR, debug=False):
     """Write the last error's traceback to the cherrypy error log."""
     cherrypy.log("", "HTTP", severity=severity, traceback=True)
 
-def log_request_headers():
+def log_request_headers(debug=False):
     """Write request headers to the cherrypy error log."""
-    h = ["  %s: %s" % (k, v) for k, v in cherrypy.request.header_list]
+    h = ["  %s: %s" % (k, v) for k, v in cherrypy.serving.request.header_list]
     cherrypy.log('\nRequest Headers:\n' + '\n'.join(h), "HTTP")
 
-def log_hooks():
+def log_hooks(debug=False):
     """Write request.hooks to the cherrypy error log."""
+    request = cherrypy.serving.request
+    
     msg = []
     # Sort by the standard points if possible.
     from cherrypy import _cprequest
     points = _cprequest.hookpoints
-    for k in cherrypy.request.hooks.keys():
+    for k in request.hooks.keys():
         if k not in points:
             points.append(k)
     
     for k in points:
         msg.append("    %s:" % k)
-        v = cherrypy.request.hooks.get(k, [])
+        v = request.hooks.get(k, [])
         v.sort()
         for h in v:
             msg.append("        %r" % h)
     cherrypy.log('\nRequest Hooks for ' + cherrypy.url() +
                  ':\n' + '\n'.join(msg), "HTTP")
 
-def redirect(url='', internal=True):
+def redirect(url='', internal=True, debug=False):
     """Raise InternalRedirect or HTTPRedirect to the given url."""
+    if debug:
+        cherrypy.log('Redirecting %sto: %s' %
+                     ({True: 'internal ', False: ''}[internal], url),
+                     'TOOLS.REDIRECT')
     if internal:
         raise cherrypy.InternalRedirect(url)
     else:
         raise cherrypy.HTTPRedirect(url)
 
-def trailing_slash(missing=True, extra=False):
+def trailing_slash(missing=True, extra=False, status=None, debug=False):
     """Redirect if path_info has (missing|extra) trailing slash."""
-    request = cherrypy.request
+    request = cherrypy.serving.request
     pi = request.path_info
     
+    if debug:
+        cherrypy.log('is_index: %r, missing: %r, extra: %r, path_info: %r' %
+                     (request.is_index, missing, extra, pi),
+                     'TOOLS.TRAILING_SLASH')
     if request.is_index is True:
         if missing:
             if not pi.endswith('/'):
                 new_url = cherrypy.url(pi + '/', request.query_string)
-                raise cherrypy.HTTPRedirect(new_url)
+                raise cherrypy.HTTPRedirect(new_url, status=status or 301)
     elif request.is_index is False:
         if extra:
             # If pi == '/', don't redirect to ''!
             if pi.endswith('/') and pi != '/':
                 new_url = cherrypy.url(pi[:-1], request.query_string)
-                raise cherrypy.HTTPRedirect(new_url)
+                raise cherrypy.HTTPRedirect(new_url, status=status or 301)
 
-def flatten():
+def flatten(debug=False):
     """Wrap response.body in a generator that recursively iterates over body.
     
     This allows cherrypy.response.body to consist of 'nested generators';
@@ -353,23 +474,28 @@ def flatten():
     """
     import types
     def flattener(input):
+        numchunks = 0
         for x in input:
             if not isinstance(x, types.GeneratorType):
+                numchunks += 1
                 yield x
             else:
                 for y in flattener(x):
-                    yield y 
-    response = cherrypy.response
+                    numchunks += 1
+                    yield y
+        if debug:
+            cherrypy.log('Flattened %d chunks' % numchunks, 'TOOLS.FLATTEN')
+    response = cherrypy.serving.response
     response.body = flattener(response.body)
 
 
-def accept(media=None):
+def accept(media=None, debug=False):
     """Return the client's preferred media-type (from the given Content-Types).
     
     If 'media' is None (the default), no test will be performed.
     
     If 'media' is provided, it should be the Content-Type value (as a string)
-    or values (as a list or tuple of strings) which the current request
+    or values (as a list or tuple of strings) which the current resource
     can emit. The client's acceptable media ranges (as declared in the
     Accept request header) will be matched in order to these Content-Type
     values; the first such string is returned. That is, the return value
@@ -391,12 +517,15 @@ def accept(media=None):
         return
     if isinstance(media, basestring):
         media = [media]
+    request = cherrypy.serving.request
     
     # Parse the Accept request header, and try to match one
     # of the requested media-ranges (in order of preference).
-    ranges = cherrypy.request.headers.elements('Accept')
+    ranges = request.headers.elements('Accept')
     if not ranges:
         # Any media type is acceptable.
+        if debug:
+            cherrypy.log('No Accept header elements', 'TOOLS.ACCEPT')
         return media[0]
     else:
         # Note that 'ranges' is sorted in order of preference
@@ -404,20 +533,28 @@ def accept(media=None):
             if element.qvalue > 0:
                 if element.value == "*/*":
                     # Matches any type or subtype
+                    if debug:
+                        cherrypy.log('Match due to */*', 'TOOLS.ACCEPT')
                     return media[0]
                 elif element.value.endswith("/*"):
                     # Matches any subtype
                     mtype = element.value[:-1]  # Keep the slash
                     for m in media:
                         if m.startswith(mtype):
+                            if debug:
+                                cherrypy.log('Match due to %s' % element.value,
+                                             'TOOLS.ACCEPT')
                             return m
                 else:
                     # Matches exact value
                     if element.value in media:
+                        if debug:
+                            cherrypy.log('Match due to %s' % element.value,
+                                         'TOOLS.ACCEPT')
                         return element.value
     
     # No suitable media-range found.
-    ah = cherrypy.request.headers.get('Accept')
+    ah = request.headers.get('Accept')
     if ah is None:
         msg = "Your client did not send an Accept header."
     else:
@@ -426,3 +563,49 @@ def accept(media=None):
             ", ".join(media))
     raise cherrypy.HTTPError(406, msg)
 
+
+class MonitoredHeaderMap(_httputil.HeaderMap):
+    
+    def __init__(self):
+        self.accessed_headers = set()
+    
+    def __getitem__(self, key):
+        self.accessed_headers.add(key)
+        return _httputil.HeaderMap.__getitem__(self, key)
+    
+    def __contains__(self, key):
+        self.accessed_headers.add(key)
+        return _httputil.HeaderMap.__contains__(self, key)
+    
+    def get(self, key, default=None):
+        self.accessed_headers.add(key)
+        return _httputil.HeaderMap.get(self, key, default=default)
+    
+    def has_key(self, key):
+        self.accessed_headers.add(key)
+        return _httputil.HeaderMap.has_key(self, key)
+
+
+def autovary(ignore=None, debug=False):
+    """Auto-populate the Vary response header based on request.header access."""
+    request = cherrypy.serving.request
+    
+    req_h = request.headers
+    request.headers = MonitoredHeaderMap()
+    request.headers.update(req_h)
+    if ignore is None:
+        ignore = set(['Content-Disposition', 'Content-Length', 'Content-Type'])
+    
+    def set_response_header():
+        resp_h = cherrypy.serving.response.headers
+        v = set([e.value for e in resp_h.elements('Vary')])
+        if debug:
+            cherrypy.log('Accessed headers: %s' % request.headers.accessed_headers,
+                         'TOOLS.AUTOVARY')
+        v = v.union(request.headers.accessed_headers)
+        v = v.difference(ignore)
+        v = list(v)
+        v.sort()
+        resp_h['Vary'] = ', '.join(v)
+    request.hooks.attach('before_finalize', set_response_header, 95)
+
