@@ -1,26 +1,100 @@
 """Session implementation for CherryPy.
 
-We use cherrypy.request to store some convenient variables as
-well as data about the session for the current request. Instead of
-polluting cherrypy.request we use a Session object bound to
-cherrypy.session to store these variables.
+You need to edit your config file to use sessions. Here's an example::
+
+    [/]
+    tools.sessions.on = True
+    tools.sessions.storage_type = "file"
+    tools.sessions.storage_path = "/home/site/sessions"
+    tools.sessions.timeout = 60
+
+This sets the session to be stored in files in the directory /home/site/sessions,
+and the session timeout to 60 minutes. If you omit ``storage_type`` the sessions
+will be saved in RAM.  ``tools.sessions.on`` is the only required line for
+working sessions, the rest are optional.
+
+By default, the session ID is passed in a cookie, so the client's browser must
+have cookies enabled for your site.
+
+To set data for the current session, use
+``cherrypy.session['fieldname'] = 'fieldvalue'``;
+to get data use ``cherrypy.session.get('fieldname')``.
+
+================
+Locking sessions
+================
+
+By default, the ``'locking'`` mode of sessions is ``'implicit'``, which means
+the session is locked early and unlocked late. If you want to control when the
+session data is locked and unlocked, set ``tools.sessions.locking = 'explicit'``.
+Then call ``cherrypy.session.acquire_lock()`` and ``cherrypy.session.release_lock()``.
+Regardless of which mode you use, the session is guaranteed to be unlocked when
+the request is complete.
+
+=================
+Expiring Sessions
+=================
+
+You can force a session to expire with :func:`cherrypy.lib.sessions.expire`.
+Simply call that function at the point you want the session to expire, and it
+will cause the session cookie to expire client-side.
+
+===========================
+Session Fixation Protection
+===========================
+
+If CherryPy receives, via a request cookie, a session id that it does not
+recognize, it will reject that id and create a new one to return in the
+response cookie. This `helps prevent session fixation attacks
+<http://en.wikipedia.org/wiki/Session_fixation#Regenerate_SID_on_each_request>`_.
+However, CherryPy "recognizes" a session id by looking up the saved session
+data for that id. Therefore, if you never save any session data,
+**you will get a new session id for every request**.
+
+================
+Sharing Sessions
+================
+
+If you run multiple instances of CherryPy (for example via mod_python behind
+Apache prefork), you most likely cannot use the RAM session backend, since each
+instance of CherryPy will have its own memory space. Use a different backend
+instead, and verify that all instances are pointing at the same file or db
+location. Alternately, you might try a load balancer which makes sessions
+"sticky". Google is your friend, there.
+
+================
+Expiration Dates
+================
+
+The response cookie will possess an expiration date to inform the client at
+which point to stop sending the cookie back in requests. If the server time
+and client time differ, expect sessions to be unreliable. **Make sure the
+system time of your server is accurate**.
+
+CherryPy defaults to a 60-minute session timeout, which also applies to the
+cookie which is sent to the client. Unfortunately, some versions of Safari
+("4 public beta" on Windows XP at least) appear to have a bug in their parsing
+of the GMT expiration date--they appear to interpret the date as one hour in
+the past. Sixty minutes minus one hour is pretty close to zero, so you may
+experience this bug as a new session id for every request, unless the requests
+are less than one second apart. To fix, try increasing the session.timeout.
+
+On the other extreme, some users report Firefox sending cookies after their
+expiration date, although this was on a system with an inaccurate system time.
+Maybe FF doesn't trust system time.
 """
 
 import datetime
 import os
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
 import random
-import sha
 import time
 import threading
 import types
 from warnings import warn
 
 import cherrypy
-from cherrypy.lib import http
+from cherrypy._cpcompat import copyitems, pickle, random20
+from cherrypy.lib import httputil
 
 
 missing = object()
@@ -28,59 +102,82 @@ missing = object()
 class Session(object):
     """A CherryPy dict-like Session object (one per request)."""
     
-    __metaclass__ = cherrypy._AttributeDocstrings
-    
     _id = None
-    id_observers = None
-    id_observers__doc = "A list of callbacks to which to pass new id's."
     
-    id__doc = "The current session ID."
+    id_observers = None
+    "A list of callbacks to which to pass new id's."
+    
     def _get_id(self):
         return self._id
     def _set_id(self, value):
         self._id = value
         for o in self.id_observers:
             o(value)
-    id = property(_get_id, _set_id, doc=id__doc)
+    id = property(_get_id, _set_id, doc="The current session ID.")
     
     timeout = 60
-    timeout__doc = "Number of minutes after which to delete session data."
+    "Number of minutes after which to delete session data."
     
     locked = False
-    locked__doc = """
+    """
     If True, this session instance has exclusive read/write access
     to session data."""
     
     loaded = False
-    loaded__doc = """
+    """
     If True, data has been retrieved from storage. This should happen
     automatically on the first attempt to access session data."""
     
     clean_thread = None
-    clean_thread__doc = "Class-level Monitor which calls self.clean_up."
+    "Class-level Monitor which calls self.clean_up."
     
     clean_freq = 5
-    clean_freq__doc = "The poll rate for expired session cleanup in minutes."
+    "The poll rate for expired session cleanup in minutes."
+    
+    originalid = None
+    "The session id passed by the client. May be missing or unsafe."
+    
+    missing = False
+    "True if the session requested by the client did not exist."
+    
+    regenerated = False
+    """
+    True if the application called session.regenerate(). This is not set by
+    internal calls to regenerate the session id."""
+    
+    debug=False
     
     def __init__(self, id=None, **kwargs):
         self.id_observers = []
         self._data = {}
         
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             setattr(self, k, v)
         
+        self.originalid = id
+        self.missing = False
         if id is None:
-            self.regenerate()
+            if self.debug:
+                cherrypy.log('No id given; making a new one', 'TOOLS.SESSIONS')
+            self._regenerate()
         else:
             self.id = id
             if not self._exists():
+                if self.debug:
+                    cherrypy.log('Expired or malicious session %r; '
+                                 'making a new one' % id, 'TOOLS.SESSIONS')
                 # Expired or malicious session. Make a new one.
                 # See http://www.cherrypy.org/ticket/709.
                 self.id = None
-                self.regenerate()
+                self.missing = True
+                self._regenerate()
     
     def regenerate(self):
         """Replace the current session (with a new id)."""
+        self.regenerated = True
+        self._regenerate()
+    
+    def _regenerate(self):
         if self.id is not None:
             self.delete()
         
@@ -102,26 +199,21 @@ class Session(object):
         """Clean up expired sessions."""
         pass
     
-    try:
-        os.urandom(20)
-    except (AttributeError, NotImplementedError):
-        # os.urandom not available until Python 2.4. Fall back to random.random.
-        def generate_id(self):
-            """Return a new session id."""
-            return sha.new('%s' % random.random()).hexdigest()
-    else:
-        def generate_id(self):
-            """Return a new session id."""
-            return os.urandom(20).encode('hex')
+    def generate_id(self):
+        """Return a new session id."""
+        return random20()
     
     def save(self):
         """Save session data."""
         try:
             # If session data has never been loaded then it's never been
-            #   accessed: no need to delete it
+            #   accessed: no need to save it
             if self.loaded:
                 t = datetime.timedelta(seconds = self.timeout * 60)
                 expiration_time = datetime.datetime.now() + t
+                if self.debug:
+                    cherrypy.log('Saving with expiry %s' % expiration_time,
+                                 'TOOLS.SESSIONS')
                 self._save(expiration_time)
             
         finally:
@@ -134,7 +226,8 @@ class Session(object):
         data = self._load()
         # data is either None or a tuple (session_data, expiration_time)
         if data is None or data[1] < datetime.datetime.now():
-            # Expired session: flush session data
+            if self.debug:
+                cherrypy.log('Expired session, flushing data', 'TOOLS.SESSIONS')
             self._data = {}
         else:
             self._data = data[0]
@@ -147,7 +240,8 @@ class Session(object):
             # clean_up is in instancemethod and not a classmethod,
             # so that tool config can be accessed inside the method.
             t = cherrypy.process.plugins.Monitor(
-                cherrypy.engine, self.clean_up, self.clean_freq * 60)
+                cherrypy.engine, self.clean_up, self.clean_freq * 60,
+                name='Session cleanup')
             t.subscribe()
             cls.clean_thread = t
             t.start()
@@ -186,7 +280,7 @@ class Session(object):
     def has_key(self, key):
         """D.has_key(k) -> True if D has a key k, else False."""
         if not self.loaded: self.load()
-        return self._data.has_key(key)
+        return key in self._data
     
     def get(self, key, default=None):
         """D.get(k[,d]) -> D[k] if k in D, else d.  d defaults to None."""
@@ -233,8 +327,8 @@ class RamSession(Session):
     def clean_up(self):
         """Clean up expired sessions."""
         now = datetime.datetime.now()
-        for id, (data, expiration_time) in self.cache.items():
-            if expiration_time < now:
+        for id, (data, expiration_time) in copyitems(self.cache):
+            if expiration_time <= now:
                 try:
                     del self.cache[id]
                 except KeyError:
@@ -254,7 +348,7 @@ class RamSession(Session):
         self.cache[self.id] = (self._data, expiration_time)
     
     def _delete(self):
-        del self.cache[self.id]
+        self.cache.pop(self.id, None)
     
     def acquire_lock(self):
         """Acquire an exclusive lock on the currently-loaded session data."""
@@ -274,13 +368,16 @@ class RamSession(Session):
 class FileSession(Session):
     """Implementation of the File backend for sessions
     
-    storage_path: the folder where session data will be saved. Each session
+    storage_path
+        The folder where session data will be saved. Each session
         will be saved as pickle.dump(data, expiration_time) in its own file;
         the filename will be self.SESSION_PREFIX + self.id.
+    
     """
     
     SESSION_PREFIX = 'session-'
     LOCK_SUFFIX = '.lock'
+    pickle_protocol = pickle.HIGHEST_PROTOCOL
     
     def __init__(self, id=None, **kwargs):
         # The 'storage_path' arg is required for file-based sessions.
@@ -296,7 +393,7 @@ class FileSession(Session):
         # The 'storage_path' arg is required for file-based sessions.
         kwargs['storage_path'] = os.path.abspath(kwargs['storage_path'])
         
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             setattr(cls, k, v)
         
         # Warn if any lock files exist at startup.
@@ -336,7 +433,7 @@ class FileSession(Session):
     def _save(self, expiration_time):
         f = open(self._get_file_path(), "wb")
         try:
-            pickle.dump((self._data, expiration_time), f)
+            pickle.dump((self._data, expiration_time), f, self.pickle_protocol)
         finally:
             f.close()
     
@@ -399,7 +496,7 @@ class FileSession(Session):
 
 class PostgresqlSession(Session):
     """ Implementation of the PostgreSQL backend for sessions. It assumes
-        a table like this:
+        a table like this::
 
             create table session (
                 id varchar(40),
@@ -410,6 +507,8 @@ class PostgresqlSession(Session):
     You must provide your own get_db function.
     """
     
+    pickle_protocol = pickle.HIGHEST_PROTOCOL
+    
     def __init__(self, id=None, **kwargs):
         Session.__init__(self, id, **kwargs)
         self.cursor = self.db.cursor()
@@ -420,7 +519,7 @@ class PostgresqlSession(Session):
         This should only be called once per process; this will be done
         automatically when using sessions.init (as the built-in Tool does).
         """
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             setattr(cls, k, v)
         
         self.db = self.get_db()
@@ -451,7 +550,7 @@ class PostgresqlSession(Session):
         return data, expiration_time
     
     def _save(self, expiration_time):
-        pickled_data = pickle.dumps(self._data)
+        pickled_data = pickle.dumps(self._data, self.pickle_protocol)
         self.cursor.execute('update session set data = %s, '
                             'expiration_time = %s where id = %s',
                             (pickled_data, expiration_time, self.id))
@@ -496,7 +595,7 @@ class MemcachedSession(Session):
         This should only be called once per process; this will be done
         automatically when using sessions.init (as the built-in Tool does).
         """
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             setattr(cls, k, v)
         
         import memcache
@@ -552,21 +651,23 @@ def save():
     
     if not hasattr(cherrypy.serving, "session"):
         return
+    request = cherrypy.serving.request
+    response = cherrypy.serving.response
     
     # Guard against running twice
-    if hasattr(cherrypy.request, "_sessionsaved"):
+    if hasattr(request, "_sessionsaved"):
         return
-    cherrypy.request._sessionsaved = True
+    request._sessionsaved = True
     
-    if cherrypy.response.stream:
+    if response.stream:
         # If the body is being streamed, we have to save the data
         #   *after* the response has been written out
-        cherrypy.request.hooks.attach('on_end_request', cherrypy.session.save)
+        request.hooks.attach('on_end_request', cherrypy.session.save)
     else:
         # If the body is not being streamed, we save the data now
         # (so we can release the lock).
-        if isinstance(cherrypy.response.body, types.GeneratorType):
-            cherrypy.response.collapse_body()
+        if isinstance(response.body, types.GeneratorType):
+            response.collapse_body()
         cherrypy.session.save()
 save.failsafe = True
 
@@ -581,29 +682,52 @@ close.priority = 90
 
 
 def init(storage_type='ram', path=None, path_header=None, name='session_id',
-         timeout=60, domain=None, secure=False, clean_freq=5, **kwargs):
+         timeout=60, domain=None, secure=False, clean_freq=5,
+         persistent=True, debug=False, **kwargs):
     """Initialize session object (using cookies).
     
-    storage_type: one of 'ram', 'file', 'postgresql'. This will be used
+    storage_type
+        One of 'ram', 'file', 'postgresql'. This will be used
         to look up the corresponding class in cherrypy.lib.sessions
         globals. For example, 'file' will use the FileSession class.
-    path: the 'path' value to stick in the response cookie metadata.
-    path_header: if 'path' is None (the default), then the response
+    
+    path
+        The 'path' value to stick in the response cookie metadata.
+    
+    path_header
+        If 'path' is None (the default), then the response
         cookie 'path' will be pulled from request.headers[path_header].
-    name: the name of the cookie.
-    timeout: the expiration timeout (in minutes) for both the cookie and
-        stored session data.
-    domain: the cookie domain.
-    secure: if False (the default) the cookie 'secure' value will not
+    
+    name
+        The name of the cookie.
+    
+    timeout
+        The expiration timeout (in minutes) for the stored session data.
+        If 'persistent' is True (the default), this is also the timeout
+        for the cookie.
+    
+    domain
+        The cookie domain.
+    
+    secure
+        If False (the default) the cookie 'secure' value will not
         be set. If True, the cookie 'secure' value will be set (to 1).
-    clean_freq (minutes): the poll rate for expired session cleanup.
+    
+    clean_freq (minutes)
+        The poll rate for expired session cleanup.
+    
+    persistent
+        If True (the default), the 'timeout' argument will be used
+        to expire the cookie. If False, the cookie will not have an expiry,
+        and the cookie will be a "session cookie" which expires when the
+        browser is closed.
     
     Any additional kwargs will be bound to the new Session instance,
     and may be specific to the storage type. See the subclass of Session
     you're using for more information.
     """
     
-    request = cherrypy.request
+    request = cherrypy.serving.request
     
     # Guard against running twice
     if hasattr(request, "_session_init_flag"):
@@ -614,6 +738,9 @@ def init(storage_type='ram', path=None, path_header=None, name='session_id',
     id = None
     if name in request.cookie:
         id = request.cookie[name].value
+        if debug:
+            cherrypy.log('ID obtained from request.cookie: %r' % id,
+                         'TOOLS.SESSIONS')
     
     # Find the storage class and call setup (first time only).
     storage_class = storage_type.title() + 'Session'
@@ -628,36 +755,57 @@ def init(storage_type='ram', path=None, path_header=None, name='session_id',
     kwargs['timeout'] = timeout
     kwargs['clean_freq'] = clean_freq
     cherrypy.serving.session = sess = storage_class(id, **kwargs)
+    sess.debug = debug
     def update_cookie(id):
         """Update the cookie every time the session id changes."""
-        cherrypy.response.cookie[name] = id
+        cherrypy.serving.response.cookie[name] = id
     sess.id_observers.append(update_cookie)
     
     # Create cherrypy.session which will proxy to cherrypy.serving.session
     if not hasattr(cherrypy, "session"):
         cherrypy.session = cherrypy._ThreadLocalProxy('session')
     
+    if persistent:
+        cookie_timeout = timeout
+    else:
+        # See http://support.microsoft.com/kb/223799/EN-US/
+        # and http://support.mozilla.com/en-US/kb/Cookies
+        cookie_timeout = None
     set_response_cookie(path=path, path_header=path_header, name=name,
-                        timeout=timeout, domain=domain, secure=secure)
+                        timeout=cookie_timeout, domain=domain, secure=secure)
 
 
 def set_response_cookie(path=None, path_header=None, name='session_id',
                         timeout=60, domain=None, secure=False):
     """Set a response cookie for the client.
     
-    path: the 'path' value to stick in the response cookie metadata.
-    path_header: if 'path' is None (the default), then the response
+    path
+        the 'path' value to stick in the response cookie metadata.
+
+    path_header
+        if 'path' is None (the default), then the response
         cookie 'path' will be pulled from request.headers[path_header].
-    name: the name of the cookie.
-    timeout: the expiration timeout for the cookie.
-    domain: the cookie domain.
-    secure: if False (the default) the cookie 'secure' value will not
+
+    name
+        the name of the cookie.
+
+    timeout
+        the expiration timeout for the cookie. If 0 or other boolean
+        False, no 'expires' param will be set, and the cookie will be a
+        "session cookie" which expires when the browser is closed.
+
+    domain
+        the cookie domain.
+
+    secure
+        if False (the default) the cookie 'secure' value will not
         be set. If True, the cookie 'secure' value will be set (to 1).
+
     """
     # Set response cookie
-    cookie = cherrypy.response.cookie
+    cookie = cherrypy.serving.response.cookie
     cookie[name] = cherrypy.serving.session.id
-    cookie[name]['path'] = (path or cherrypy.request.headers.get(path_header)
+    cookie[name]['path'] = (path or cherrypy.serving.request.headers.get(path_header)
                             or '/')
     
     # We'd like to use the "max-age" param as indicated in
@@ -666,7 +814,8 @@ def set_response_cookie(path=None, path_header=None, name='session_id',
     # the browser. So we have to use the old "expires" ... sigh ...
 ##    cookie[name]['max-age'] = timeout * 60
     if timeout:
-        cookie[name]['expires'] = http.HTTPDate(time.time() + (timeout * 60))
+        e = time.time() + (timeout * 60)
+        cookie[name]['expires'] = httputil.HTTPDate(e)
     if domain is not None:
         cookie[name]['domain'] = domain
     if secure:
@@ -675,10 +824,9 @@ def set_response_cookie(path=None, path_header=None, name='session_id',
 
 def expire():
     """Expire the current session cookie."""
-    name = cherrypy.request.config.get('tools.sessions.name', 'session_id')
+    name = cherrypy.serving.request.config.get('tools.sessions.name', 'session_id')
     one_year = 60 * 60 * 24 * 365
-    exp = time.gmtime(time.time() - one_year)
-    t = time.strftime("%a, %d-%b-%Y %H:%M:%S GMT", exp)
-    cherrypy.response.cookie[name]['expires'] = t
+    e = time.time() - one_year
+    cherrypy.serving.response.cookie[name]['expires'] = httputil.HTTPDate(e)
 
 
